'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

var View = 'div';

var Text = 'span';

var Button = 'button';

var defaults = {
  View: View,
  Text: Text,
  Button: Button
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

if (!React.createContext) {
  throw new Error('React.createContext not defined. Try using react > 16');
}

var _React$createContext = React.createContext(_extends({}, defaults)),
    Provider = _React$createContext.Provider,
    Consumer = _React$createContext.Consumer;

var ComponentsProvider = function (_React$PureComponent) {
  inherits(ComponentsProvider, _React$PureComponent);

  function ComponentsProvider() {
    var _temp, _this, _ret;

    classCallCheck(this, ComponentsProvider);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, _React$PureComponent.call.apply(_React$PureComponent, [this].concat(args))), _this), _this.state = {
      components: _extends({}, defaults, _this.props.components)
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  ComponentsProvider.prototype.render = function render() {
    var children = this.props.children;
    var components = this.state.components;


    if (!Provider) {
      throw new Error('Provider not defined. Try using react > 16');
    }

    return React.createElement(
      Provider,
      { value: components },
      children
    );
  };

  return ComponentsProvider;
}(React.PureComponent);

function withComponents(WrappedComponent) {
  var WithComponents = function (_React$PureComponent) {
    inherits(WithComponents, _React$PureComponent);

    function WithComponents() {
      classCallCheck(this, WithComponents);
      return possibleConstructorReturn(this, _React$PureComponent.apply(this, arguments));
    }

    WithComponents.prototype.render = function render() {
      var _props = this.props,
          components = _props.components,
          passedProps = objectWithoutProperties(_props, ['components']);


      if (!Consumer) {
        throw new Error('Consumer not defined. Try using react > 16');
      }

      return React.createElement(
        Consumer,
        null,
        function (contextComponents) {
          var passedComponents = _extends({}, defaults, contextComponents, components);
          return React.createElement(WrappedComponent, _extends({}, passedProps, {
            components: passedComponents
          }));
        }
      );
    };

    return WithComponents;
  }(React.PureComponent);

  WithComponents.displayName = 'WithComponents(' + getDisplayName(WrappedComponent) + ')';
  return WithComponents;
}

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}

/* export */var ComponentsProvider$1 = ComponentsProvider;
/* export */var withComponents$1 = withComponents;

var index = {
  ComponentsProvider: ComponentsProvider$1,
  withComponents: withComponents$1
};

module.exports = index;
